"use strict";
var Ansi_Codes = require("../Ansi.Codes/index.js");
var Ansi_Output = require("../Ansi.Output/index.js");
var Control_Applicative = require("../Control.Applicative/index.js");
var Data_Array = require("../Data.Array/index.js");
var Data_Eq = require("../Data.Eq/index.js");
var Data_Foldable = require("../Data.Foldable/index.js");
var Data_Functor = require("../Data.Functor/index.js");
var Data_Maybe = require("../Data.Maybe/index.js");
var Data_Monoid = require("../Data.Monoid/index.js");
var Data_Newtype = require("../Data.Newtype/index.js");
var Data_Result = require("../Data.Result/index.js");
var Data_String_CodePoints = require("../Data.String.CodePoints/index.js");
var Data_Traversable = require("../Data.Traversable/index.js");
var Data_Tuple = require("../Data.Tuple/index.js");
var Data_Unit = require("../Data.Unit/index.js");
var Effect = require("../Effect/index.js");
var Effect_Class = require("../Effect.Class/index.js");
var Effect_Class_Console = require("../Effect.Class.Console/index.js");
var Node_Encoding = require("../Node.Encoding/index.js");
var Node_FS_Async = require("../Node.FS.Async/index.js");
var Node_FS_Stats = require("../Node.FS.Stats/index.js");
var Node_FS_Sync = require("../Node.FS.Sync/index.js");
var Node_Path = require("../Node.Path/index.js");
var Node_Process = require("../Node.Process/index.js");
var Transity_Data_Ledger = require("../Transity.Data.Ledger/index.js");
var Transity_Plot = require("../Transity.Plot/index.js");
var Transity_Utils = require("../Transity.Utils/index.js");

// TODO: Move validation to parsing
var utcError = "All transfers or their parent transaction must have a valid UTC field";
var usageString = "\x0aUsage: transity <command> <path/to/journal.yaml>\x0a\x0aCommand             Description\x0a------------------  ------------------------------------------------------------\x0abalance             Simple balance of all accounts\x0atransactions        All transactions and their transfers\x0atransfers           All transfers with one transfer per line\x0aentries             All individual deposits & withdrawals, space separated\x0aledger-entries      All entries in Ledger format\x0acsv                 Entries, comma separated\x0atsv                 Entries, tab separated\x0aentries-by-account  All individual deposits & withdrawals, grouped by account\x0agplot               Code and data for gnuplot impulse diagram\x0a                      to visualize transfers of all accounts\x0agplot-cumul         Code and data for cumuluative gnuplot step chart\x0a                      to visualize balance of all accounts\x0aunused-files        Recursively list all files in a directory\x0a                      which are not referenced in the journal\x0ahelp                Print this help dialog\x0aversion             Print currently used version\x0a";
var run = function (command) {
    return function (filePathRel) {
        return function (ledger) {
            if (command === "balance") {
                return Data_Result.Ok.create(Transity_Data_Ledger.showBalance(Transity_Utils.ColorYes.value)(ledger));
            };
            if (command === "transactions") {
                return Data_Result.Ok.create(Transity_Data_Ledger.showPrettyAligned(Transity_Utils.ColorYes.value)(ledger));
            };
            if (command === "transfers") {
                return Data_Result.Ok.create(Transity_Data_Ledger.showTransfers(Transity_Utils.ColorYes.value)(ledger));
            };
            if (command === "entries") {
                return Data_Result.note(utcError)(Transity_Data_Ledger.showEntries(" ")(ledger));
            };
            if (command === "ledger-entries") {
                return Data_Result.Ok.create(Transity_Data_Ledger.entriesToLedger(ledger));
            };
            if (command === "csv") {
                return Data_Result.note(utcError)(Transity_Data_Ledger.showEntries(",")(ledger));
            };
            if (command === "tsv") {
                return Data_Result.note(utcError)(Transity_Data_Ledger.showEntries("\x09")(ledger));
            };
            if (command === "entries-by-account") {
                return Data_Result.note(utcError)(Transity_Data_Ledger.showEntriesByAccount(ledger));
            };
            if (command === "gplot") {
                return Data_Functor.mapFlipped(Data_Result.functorResult)(Data_Result.note(utcError)(Transity_Data_Ledger.showEntriesByAccount(ledger)))(function (entries) {
                    return Transity_Plot.gplotCode(Data_Newtype.over(Transity_Plot.newtypeGplotConfig)(Transity_Plot.newtypeGplotConfig)(Transity_Plot.GplotConfig)(function (v) {
                        return {
                            data: entries,
                            title: filePathRel,
                            imgHeight: v.imgHeight,
                            imgWidth: v.imgWidth,
                            labelY: v.labelY,
                            lineColor: v.lineColor,
                            terminalType: v.terminalType
                        };
                    })(Transity_Plot.configDefault));
                });
            };
            if (command === "gplot-cumul") {
                return Data_Functor.mapFlipped(Data_Result.functorResult)(Data_Result.note(utcError)(Transity_Data_Ledger.showEntriesByAccount(ledger)))(function (entries) {
                    return Transity_Plot.gplotCodeCumul(Data_Newtype.over(Transity_Plot.newtypeGplotConfig)(Transity_Plot.newtypeGplotConfig)(Transity_Plot.GplotConfig)(function (v) {
                        return {
                            data: entries,
                            title: filePathRel + " - Cumulative",
                            imgHeight: v.imgHeight,
                            imgWidth: v.imgWidth,
                            labelY: v.labelY,
                            lineColor: v.lineColor,
                            terminalType: v.terminalType
                        };
                    })(Transity_Plot.configDefault));
                });
            };
            return new Data_Result["Error"]("\"" + (command + "\" is not a valid command"));
        };
    };
};
var getAllFiles = function (directoryPath) {
    var addFiles = function (dirPath) {
        return function __do() {
            var entriesRel = Node_FS_Sync.readdir(dirPath)();
            var cleanEntriesRel = Data_Array.filter(function (v) {
                return v !== ".DS_Store";
            })(entriesRel);
            var entriesAbs = Data_Functor.mapFlipped(Data_Functor.functorArray)(cleanEntriesRel)(function (entry) {
                return dirPath + ("/" + entry);
            });
            var statEntries = Data_Traversable.sequence(Data_Traversable.traversableArray)(Effect.applicativeEffect)(Data_Functor.mapFlipped(Data_Functor.functorArray)(entriesAbs)(Node_FS_Sync.stat))();
            var pathStatsTuples = Data_Array.zip(entriesAbs)(statEntries);
            var fileTuples = Data_Array.filter(function (tuple) {
                return Node_FS_Stats.isFile(Data_Tuple.snd(tuple));
            })(pathStatsTuples);
            var files = Data_Functor.mapFlipped(Data_Functor.functorArray)(fileTuples)(Data_Tuple.fst);
            var dirTuples = Data_Array.filter(function (tuple) {
                return Node_FS_Stats.isDirectory(Data_Tuple.snd(tuple));
            })(pathStatsTuples);
            var $12 = Data_Array["null"](dirTuples);
            if ($12) {
                return files;
            };
            var filesNested = Data_Traversable.sequence(Data_Traversable.traversableArray)(Effect.applicativeEffect)(Data_Functor.mapFlipped(Data_Functor.functorArray)(dirTuples)(function (v) {
                return addFiles(v.value0);
            }))();
            return Data_Array.concat(Data_Array.cons(files)(filesNested));
        };
    };
    return addFiles(directoryPath);
};
var config = {
    colorState: Transity_Utils.ColorYes.value
};
var errorAndExit = function (message) {
    return function __do() {
        Effect_Class_Console.error(Effect_Class.monadEffectEffect)((function () {
            var $16 = Data_Eq.eq(Transity_Utils.eqColorFlag)(config.colorState)(Transity_Utils.ColorYes.value);
            if ($16) {
                return Ansi_Output.withGraphics(Ansi_Output.foreground(Ansi_Codes.Red.value))(message);
            };
            return message;
        })())();
        return Node_Process.exit(1)();
    };
};

// | Asynchronously logs all non existent referenced files
var checkFilePaths = function (ledgerFilePath) {
    return function (v) {
        var files = Data_Foldable.foldMap(Data_Foldable.foldableArray)(Data_Monoid.monoidArray)(function (v1) {
            return v1.files;
        })(v.transactions);
        return function __do() {
            Data_Foldable.for_(Effect.applicativeEffect)(Data_Foldable.foldableArray)(files)(function (filePathRel) {
                return function __do() {
                    var filePathAbs = Node_Path.resolve([ ledgerFilePath ])(filePathRel)();
                    return Node_FS_Async.exists(filePathAbs)(function (doesExist) {
                        if (doesExist) {
                            return Control_Applicative.pure(Effect.applicativeEffect)(Data_Unit.unit);
                        };
                        return Effect_Class_Console.log(Effect_Class.monadEffectEffect)(Ansi_Output.withGraphics(Ansi_Output.foreground(Ansi_Codes.Yellow.value))("Warning: \"" + (filePathAbs + "\" does not exist")));
                    })();
                };
            })();
            return new Data_Result.Ok("");
        };
    };
};
var loadAndExec = function (v) {
    return function (v1) {
        if (v1.length === 2) {
            return function __do() {
                var filePathAbs = Node_Path.resolve([ v ])(v1[1])();
                var ledgerFileContent = Node_FS_Sync.readTextFile(Node_Encoding.UTF8.value)(filePathAbs)();
                var v2 = Transity_Data_Ledger.fromYaml(ledgerFileContent);
                if (v2 instanceof Data_Result["Error"]) {
                    return new Data_Result["Error"](v2.value0);
                };
                if (v2 instanceof Data_Result.Ok) {
                    var journalDir = (function () {
                        var $26 = Data_Eq.eq(Data_Maybe.eqMaybe(Data_Eq.eqInt))(Data_String_CodePoints.indexOf("/dev/fd/")(filePathAbs))(new Data_Maybe.Just(0));
                        if ($26) {
                            return v;
                        };
                        return Node_Path.dirname(filePathAbs);
                    })();
                    checkFilePaths(journalDir)(v2.value0)();
                    return run(v1[0])(v1[1])(v2.value0);
                };
                throw new Error("Failed pattern match at Main (line 129, column 3 - line 138, column 44): " + [ v2.constructor.name ]);
            };
        };
        return Control_Applicative.pure(Effect.applicativeEffect)(new Data_Result["Error"]("loadAndExec expects an array with length 2"));
    };
};
var main = function __do() {
    var $$arguments = Node_Process.argv();
    var $30 = [ Data_Array.index($$arguments)(2), Data_Array.index($$arguments)(3), Data_Array.index($$arguments)(4) ];
    if ($30.length === 3 && ($30[0] instanceof Data_Maybe.Just && ($30[0].value0 === "help" && ($30[1] instanceof Data_Maybe.Nothing && $30[2] instanceof Data_Maybe.Nothing)))) {
        return Effect_Class_Console.log(Effect_Class.monadEffectEffect)(usageString)();
    };
    if ($30.length === 3 && ($30[0] instanceof Data_Maybe.Just && ($30[0].value0 === "version" && ($30[1] instanceof Data_Maybe.Nothing && $30[2] instanceof Data_Maybe.Nothing)))) {
        return Effect_Class_Console.log(Effect_Class.monadEffectEffect)("0.8.0")();
    };
    if ($30.length === 3 && ($30[0] instanceof Data_Maybe.Just && ($30[1] instanceof Data_Maybe.Nothing && $30[2] instanceof Data_Maybe.Nothing))) {
        return errorAndExit("No path to a ledger file was provided\x0a\x0a" + usageString)();
    };
    if ($30.length === 3 && ($30[0] instanceof Data_Maybe.Just && ($30[0].value0 === "unused-files" && ($30[1] instanceof Data_Maybe.Just && $30[2] instanceof Data_Maybe.Just)))) {
        var ledgerFilePathAbs = Node_Path.resolve([  ])($30[2].value0)();
        var ledgerFileContent = Node_FS_Sync.readTextFile(Node_Encoding.UTF8.value)(ledgerFilePathAbs)();
        var v = Transity_Data_Ledger.fromYaml(ledgerFileContent);
        if (v instanceof Data_Result["Error"]) {
            return errorAndExit(v.value0)();
        };
        if (v instanceof Data_Result.Ok) {
            var currentDir = Node_Process.cwd();
            var journalDir = (function () {
                var $45 = Data_Eq.eq(Data_Maybe.eqMaybe(Data_Eq.eqInt))(Data_String_CodePoints.indexOf("/dev/fd/")(ledgerFilePathAbs))(new Data_Maybe.Just(0));
                if ($45) {
                    return currentDir;
                };
                return Node_Path.dirname(ledgerFilePathAbs);
            })();
            checkFilePaths(journalDir)(v.value0)();
            var filesDir = Node_Path.resolve([  ])($30[1].value0)();
            var foundFiles = getAllFiles(filesDir)();
            var ledgerFilesRel = Data_Foldable.foldMap(Data_Foldable.foldableArray)(Data_Monoid.monoidArray)(function (v1) {
                return v1.files;
            })(v.value0.transactions);
            var ledgerFilesAbs = Data_Traversable.sequence(Data_Traversable.traversableArray)(Effect.applicativeEffect)(Data_Functor.mapFlipped(Data_Functor.functorArray)(ledgerFilesRel)(function (fileRel) {
                return Node_Path.resolve([ journalDir ])(fileRel);
            }))();
            return Data_Foldable.for_(Effect.applicativeEffect)(Data_Foldable.foldableArray)(Data_Array.difference(Data_Eq.eqString)(foundFiles)(ledgerFilesAbs))(function (filePathAbs) {
                return Effect_Class_Console.log(Effect_Class.monadEffectEffect)(Ansi_Output.withGraphics(Ansi_Output.foreground(Ansi_Codes.Yellow.value))("Warning: \"" + (filePathAbs + "\" is not referenced in the journal")));
            })();
        };
        throw new Error("Failed pattern match at Main (line 194, column 7 - line 218, column 60): " + [ v.constructor.name ]);
    };
    if ($30.length === 3 && ($30[0] instanceof Data_Maybe.Just && ($30[1] instanceof Data_Maybe.Just && $30[2] instanceof Data_Maybe.Nothing))) {
        var currentDir = Node_Process.cwd();
        var result = loadAndExec(currentDir)([ $30[0].value0, $30[1].value0 ])();
        if (result instanceof Data_Result.Ok) {
            return Effect_Class_Console.log(Effect_Class.monadEffectEffect)(result.value0)();
        };
        if (result instanceof Data_Result["Error"]) {
            return errorAndExit(result.value0)();
        };
        throw new Error("Failed pattern match at Main (line 225, column 7 - line 227, column 46): " + [ result.constructor.name ]);
    };
    return Effect_Class_Console.log(Effect_Class.monadEffectEffect)(usageString)();
};
module.exports = {
    config: config,
    usageString: usageString,
    utcError: utcError,
    run: run,
    checkFilePaths: checkFilePaths,
    loadAndExec: loadAndExec,
    errorAndExit: errorAndExit,
    getAllFiles: getAllFiles,
    main: main
};
